; Procedure IsFloppy to check if floppy drive is ready
; Entry conditions: drive number passed from Clipper, A=0, B=1
; Exit conditions: 0 = drive ready, 1 = write protected drive,
; 2 = other drive errors, 3 = invalid drive number passed to procedure
; when linking with EXOSPACE, must add EXOSPACE PACKAGE DOS25 to link script
; Programmer:   Paul Bosselaers
; Date:         8/3/94

.model large
.286

include \clipper5\include\extasm.inc

PUBLIC IsFloppy

.data
buffer  db      512 dup(0)

.code
IsFloppy        proc    far
; save registers
        push    bp
        mov     bp,sp
        push    ds
        push    es
        push    di
        push    si
        push    dx
        push    cx
        push    bx

; get drive number from Clipper
        mov     ax,1
        push    ax
        call    __parni                         ; returns number in ax
        add     sp,2

; check for valid number, 0 or 1
        push    ax                              ; save drive num for later
        or      al,al                           ; is it 0?
        je      read_drive                      ; yes, go for it
        cmp     al,1                            ; is it 1?
        je      read_drive                      ; yes, go for it

; invalid number so exit        
        mov     ax,0003h
        jmp     short send_code

; low level read of drive
read_drive:
        clc
        mov     bx,@data                        ; look at data segment
        mov     ds,bx
        mov     bx, offset buffer               ; buffer to receive sector
        mov     cx,1                            ; read 1 sector
        xor     dx,dx                           ; start at sector 0
        int     25h                             ; read it
; int 25h pushes flags onto stack, so adjust for it
        pop     dx
        pop     ax                              ; get back drive number
        jc      bad_exit                        ; error reading, so exit

; now check if write protected
        xor     dx,dx                           ; start at sector 0
        int     26h                             ; write sector back to disk 
; int 26h pushes flags onto stack, so adjust for it
        pop     dx
        jnc     good_exit                       ; no error

; check for error type
        or      al,al                           ; write protect? (al=0)
        je      write_prot                      ; yes, write protect error
bad_exit:
        mov     ax,0002h                        ; code for other error
        jmp     short send_code

write_prot:
        mov     ax,0001h
        jmp     short send_code

good_exit:
        xor     ax,ax                           ; set it to zero

send_code:
; send exit code to Clipper
        push    ax
        call    __retni
        add     sp,2

; restore registers
        pop     bx
        pop     cx
        pop     dx
        pop     si
        pop     di
        pop     es
        pop     ds
        pop     bp
        ret

IsFloppy        endp
end


