; function KBDSTAT to check status of Num Lock, Caps Lock settings,
;       Alt and Ctrl key presses
; Entry conditions: logical fields for each passed by reference
; param 1 is Num-Lock, param 2 is Caps-Lock, param 3 is Alt, param 4 is Ctrl
; changes params as appropriate
; Returns Logical True if no error in params, Logical False if bad params
; Programmer: Paul Bosselaers
; Date:       2/08/95, updated to add alt & ctrl fields 5/22/96

.model large
.286

;include \clipper5\include\extasm.inc            ; extend definitions
        extrn   __parinfo:far
        extrn   __storl:far
        extrn   __retl:far

PUBLIC KBDSTAT

.code
KBDSTAT      proc    far
; save registers as required
        push    bp
        mov     bp,sp
        push    ds
        push    si
        push    di
        push    bx

; check parameters
        xor     ax,ax                   ; 1st check number of params
        push    ax
        call    __parinfo
        add     sp,2
;        cmp     ax,2                    ; need 2 params.
        cmp     ax,4                    ; need 4 params.
        jne     bad_exit                ; incorrect number

; check param types
        mov     ax,0001h                ; 1st param
        push    ax
        call    __parinfo
        add     sp,2
        cmp     ax,0024h                ; check if logical and by reference
        jne     bad_exit                ; not the correct type for #1
         
        mov     ax,0002h                ; 2nd param
        push    ax
        call    __parinfo
        add     sp,2
        cmp     ax,0024h                ; check if logical and by reference
        jne     bad_exit                ; not the correct type for #2

        mov     ax,0003h                ; 3rd param
        push    ax
        call    __parinfo
        add     sp,2
        cmp     ax,0024h                ; check if logical and by reference
        jne     bad_exit                ; not the correct type for #3

        mov     ax,0004h                ; 4th param
        push    ax
        call    __parinfo
        add     sp,2
        cmp     ax,0024h                ; check if logical and by reference
        jne     bad_exit                ; not the correct type for #4

; check the hardware for num lock status
        push    ds                      ; save ds
        mov     bx,40h                  ; point DS to the BIOS data area
        mov     ds,bx                   ; 
        mov     bx,17h                  ; point to keyboard status flags
        test    byte ptr [bx], 020h     ; is num lock on?

; set param #1
        pop     ds                      ; restore ds
        mov     ax,0001h                ; param 1
        push    ax
        jz      no_nums                 ; num lock is off
        mov     ax,0001h                ; 1 = TRUE
        jmp     short send1

no_nums:
        xor     ax,ax                   ; 0 = FALSE

send1:
        push    ax
        call    __storl
        add     sp,4

; check the hardware for caps lock status
        push    ds                      ; save ds
        mov     bx,40h                  ; point DS to the BIOS data area
        mov     ds,bx                   ; 
        mov     bx,17h                  ; point to keyboard status flags
        test    byte ptr [bx], 040h     ; is caps lock on?

; set param #2
        pop     ds                      ; restore ds
        mov     ax,0002h                ; param 2
        push    ax
        jz      no_caps                 ; caps lock is off
        mov     ax,0001h                ; 1 = TRUE
        jmp     short send2

no_caps:
        xor     ax,ax                   ; 0 = FALSE

send2:
        push    ax
        call    __storl
        add     sp,4

; check the hardware for alt key status
        push    ds                      ; save ds
        mov     bx,40h                  ; point DS to the BIOS data area
        mov     ds,bx                   ; 
        mov     bx,17h                  ; point to keyboard status flags
        test    byte ptr [bx], 08h      ; was alt key pressed?

; set param #3
        pop     ds                      ; restore ds
        mov     ax,0003h                ; param 3
        push    ax
        jz      no_alt                  ; alt not pressed
        mov     ax,0001h                ; 1 = TRUE
        jmp     short send3

no_alt:
        xor     ax,ax                   ; 0 = FALSE

send3:
        push    ax
        call    __storl
        add     sp,4

; check the hardware for ctrl key status
        push    ds                      ; save ds
        mov     bx,40h                  ; point DS to the BIOS data area
        mov     ds,bx                   ; 
        mov     bx,17h                  ; point to keyboard status flags
        test    byte ptr [bx], 04h      ; was ctrl key pressed?

; set param #4
        pop     ds                      ; restore ds
        mov     ax,0004h                ; param 4
        push    ax
        jz      no_ctrl                 ; ctrl not pressed
        mov     ax,0001h                ; 1 = TRUE
        jmp     short send4

no_ctrl:
        xor     ax,ax                   ; 0 = FALSE

send4:
        push    ax
        call    __storl
        add     sp,4

good_exit:
        mov     ax,0001h
        jmp     short send_code

bad_exit:
        xor     ax,ax                   ; 0 = FALSE
        
send_code:
        push    ax
        call    __retl                  ; return code to Clipper
        add     sp,2

;restore registers
        pop     bx
        pop     di
        pop     si
        pop     ds
        pop     bp
        ret
KBDSTAT      endp
end


